#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil

import nysol.mcmd as nm
import nysol.util as nu
import nysol.util.margs as margs

from nysol.util.mtemp import Mtemp
from nysol.util.mmkdir import mkDir
from nysol.util.mparallel import meach as meach
from nysol.util.mrecount import mrecount


class mccomp(object):

	helpMSG="""
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) 連結成分を出力する

用法) #{$cmd} ei= ef= [ni=] [nf=] [o=] [T=] [-verbose] [-mcmdenv] [--help]

  ファイル名指定
  ei=    : 枝データファイル
  ef=    : 枝データ上の2つの節点項目名(省略時は"node1,node2")
  ni=    : 節点データファイル
  nf=    : 節点データ上の節点項目名(省略時は"node")
  o=     : 出力ファイル(連結成分ID-枝:-nodeを指定することでクリークID-節点に変更可能,省略時は標準出力)

  その他
  T= : ワークディレクトリ(default:/tmp)
  -verbose : Rの実行ログを出力
  --help : ヘルプの表示

入力形式)
一般グラフを節点ペアで表現した形式。

o=の出力形式)
節点と連結成分IDを出力する。
出力項目は"id,node,size"の3項目である。
sizeは連結成分を構成する節点数である。

例)
$ cat data/edge.csv 
n1,n2
a,d
a,e
b,f
d,e
f,g
g,b
g,h

$ #{$cmd} ei=edge.csv ef=n1,n2 o=output.csv
##END# #{$cmd} ei=edge.csv ef=n1,n2 -node o=output.csv
$ cat output.csv 
id%0,node,size
1,a,3
1,d,3
1,e,3
2,b,4
2,f,4
2,g,4
2,h,4

例) 節点ファイルも指定した例
$ cat node.csv 
n
a
b
c
d
e
f
g
h

$ #{$cmd} ei=edge.csv ef=n1,n2 ni=node.csv nf=n o=output.csv
#END# #{$cmd} ei=edge.csv o=output.csv ef=n1,n2 ni=node.csv nf=n
$ cat output.csv 
id%0,node,size
1,a,3
1,d,3
1,e,3
2,b,4
2,f,4
2,g,4
2,h,4
3,c,1
4,i,1

# Copyright(c) NYSOL 2012- All Rights Reserved.
"""

	verInfo="version=0.1"

	paramter = {	
		"ei":"str",
		"ef":"str",
		"ni":"str",
		"nf":"str",
		"o":"str",
		"rp":"bool",
		"verbose":"bool"
	}
	paramcond = {	
		"hissu": ["ei","ef"]
	}	

	def help():
		print(mccomp.helpMSG) 

	def ver():
		print(mccomp.verInfo)

	def __param_check_set(self , kwd):

		# 存在チェック
		for k,v in kwd.items():
			if not k in mccomp.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )

		self.msgoff = True

		self.oFile  = kwd["o"] if "o" in kwd else None

		self.ei  = kwd["ei"] 
		ef0 = kwd["ef"].split(",")
		self.ef1 = ef0[0]
		self.ef2 = ef0[1] 

		self.ni = kwd["ni"] if "ni" in kwd else None
		self.nf = kwd["nf"] if "nf" in kwd else None
		self.verbose  = kwd["verbose"]  if "verbose" in kwd else False
		self.rpf  = kwd["rp"] if "rp" in kwd else False 



	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 

	def __init__(self,**kwd):
		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)



	####
	# generating the R script for graph features
	# pars: parameters for each graph feature
	def genRscript(self,eFile,oFile,cidFile,scpFile):
		
		r_proc = '''
library(igraph)
## reading edge file
g=read.graph("{eFile}",format="edgelist",directed=FALSE)
c=components(g)
seq=0:(length(c$membership)-1)
dat=data.frame(id=c$membership,nid=seq,size=c$csize[c$membership])
write.csv(dat,file="{oFile}",quote=FALSE,row.names = FALSE)
write.table(max(c$membership),file="{cidFile}",col.names = FALSE,row.names = FALSE)
		'''.format( eFile = eFile ,oFile = oFile,cidFile = cidFile)

		with open(scpFile,"w") as fpw:
			fpw.write(r_proc)


	def conv2num(self,ei,ni,ef1,ef2,nf,numFile,mapFile,isoFile):
		#MCMD::msgLog("converting graph files into a pair of numbered nodes ...")
		#wf=MCMD::Mtemp.new
		#wf1=wf.file
		#wf2=wf.file
		#wf3=wf.file
		
		allinObj =[]		
		
		
		inobj1 = nm.mcut(f="%s:node"%(ef1),i=ei )
		inobj2 = nm.mcut(f="%s:node"%(ef2),i=ei )

		allinObj.append(inobj1)
		allinObj.append(inobj2)

	
		if nf :
			allinObj.append(nm.mcut(f="%s:node"%(nf),i=ni) )
		
		#nodes list that are included in edge
		ne_list =  nm.muniq(k="node",i=[inobj1,inobj2]).msetstr(v=1,a="eNode")
		
		# isolate nodes list
		iso_f = nm.mcommon(i=allinObj ,k="node",m=ne_list,r=True).mcut(f="node",o=isoFile)
		iso_f.run()

		# create a mapping table between the original node label and the number iGraph will use
		map_f	=   nm.muniq(i=allinObj , k="node" )
		map_f	<<= nm.mjoin(m=ne_list , k="node" ,f="eNode" )
		map_f	<<= nm.mnullto( f="eNode",v=0 )
		map_f <<= nm.mnumber(s="eNode%r,node",a="nid",o=mapFile) 
		map_f.run()
		
		# create a data file that R script read
		f=None
		f <<= nm.mjoin(k=ef1,K="node",m=mapFile,f="nid:nid1",i=ei)
		f <<= nm.mjoin(k=ef2,K="node",m=mapFile,f="nid:nid2")
		f <<= nm.mcut(f="nid1,nid2",nfno=True)
		f <<= nm.cmd("tr ',' ' ' " )
		f <<= nm.mwrite(o=numFile)
		f.run()



	# ============
	# entry point
	def run(self,**kw_args):
	
		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		# convert the original graph to one igraph can handle
		temp=Mtemp()
		numFile = temp.file()
		mapFile = temp.file()
		isoFile = temp.file()
		cluFile = temp.file()
		cidFile = temp.file()
		scpFile = temp.file()


		self.conv2num(self.ei,self.ni,self.ef1,self.ef2,self.nf,numFile,mapFile,isoFile)
		self.genRscript(numFile,cluFile,cidFile,scpFile)

		if self.verbose :
			os.system("R --vanilla -q < %s"%(scpFile))
		else:
			os.system("R --vanilla -q --slave < %s 2>/dev/null "%(scpFile))
			
		cid=0
		with open(cidFile) as rfp:
			xxx = rfp.read()
			cid = int(xxx.rstrip())

		isc = None
		isc <<= nm.mnumber(s="node", S=cid+1 ,a="id",i=isoFile)
		isc <<= nm.msetstr(v=1,a="size")
		isc <<= nm.mcut(f="id,node,size")
		if self.nf:
			isc <<= nm.mfldname(f="node:"+self.nf) 

		# #{cluFile}
		# id,nid,size
		# 1,0,3
		# 2,1,4
		# 1,2,3
		# 1,3,3
		# 2,4,4
		# 2,5,4
		# 2,6,4
		cln = None
		cln <<= nm.mjoin(k="nid",m=mapFile,i=cluFile,f="node")
		cln <<= nm.mcut(f="id,node,size")
		if self.nf:
			cln <<= nm.mfldname(f="node:"+self.nf) 

		nm.msortf(f="id",i=[isc,cln],o=self.oFile,rp=self.rpf).run()

		nu.mmsg.endLog(self.__cmdline())




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mstdout(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd = n_core.getparalist("mstdout",3)
	_disabled_ouputlist = True
	def __init__(self,*args, **kw_args) :
		super(Nysol_Mstdout,self).__init__("mstdout",nutil.args2dict(args,kw_args,Nysol_Mstdout._kwd))

def mstdout(self,*args, **kw_args):
	return Nysol_Mstdout(nutil.args2dict(args,kw_args,Nysol_Mstdout._kwd)).addPre(self)


setattr(NysolMOD_CORE, "mstdout", mstdout)



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Writecsv(NysolMOD_CORE):

	_kwd,_inkwd,_outkwd = n_core.getparalist("writecsv",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Writecsv,self).__init__("writecsv",nutil.args2dict(args,kw_args,Nysol_Writecsv._kwd,uk="o"))


def writecsv(self,*args, **kw_args):
	return Nysol_Writecsv(nutil.args2dict(args,kw_args,Nysol_Writecsv._kwd,uk="o")).addPre(self)

setattr(NysolMOD_CORE, "writecsv", writecsv)




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Readcsv(NysolMOD_CORE): # i=必須にする？ #err処理は？

	_kwd ,_inkwd,_outkwd = n_core.getparalist("readcsv",3)

	def __init__(self,*args, **kw_args) :
		if len(args) != 1 :
			print("arge only one")
			return None

		if isinstance(args[0],str) :
			kw_args["i"] = args[0]
		elif isinstance(args[0],list) :
			kw_args["i"] = ','.join(args[0])
		else :
			print("unsuport type")
			return None
		super(Nysol_Readcsv,self).__init__("readcsv",nutil.args2dict((),kw_args,Nysol_Readcsv._kwd))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#require "rubygems"
#require "nysol/mcmd"
#require "nysol/take"

#require "nysol/traDB.rb"

#module TAKE
import os
import shutil
import nysol.mcmd as nm
import nysol.util.mtemp as mtemp
from nysol.take import extcore as extTake
import nysol.vsop._vsoplib as VSOP

#========================================================================
# 列挙関数:lcm 利用DB:TraDB
#========================================================================
class LcmEp(object):
	#未実装zddつくってから
	def reduceTaxo(self,pat,items):
		#tf=mtemp.Mtemp()

		if items.taxonomy==None:
			return pat

		#xxrt = tf.file
		taxo=items.taxonomy
		xxrt = nm.mtrafld(f=taxo.itemFN+","+taxo.taxoFN ,valOnly=True,a="__fld",i=taxo.file).mcut(f="__fld")

		# xxrtの内容：oyakoに親子関係にあるアイテム集合のリストが格納される
		# __fld
		# A X
		# B X
		# C Y
		# D Z
		# E Z
		# F Z
		oyako=VSOP.constant(0)

		for fldVal in xxrt:
			oyako=oyako+VSOP.itemset(fldVal[0])
		# 親子リストにあるアイテム集合を含むパターンを削除する
		pat=pat.restrict(oyako).iif(0,pat)

		return pat



	intMax=2147483646

	# posトランザクションの重み計算
	# マニュアルの式(10)
	def calOmega(self,posCnt):
		return LcmEp.intMax/posCnt

	# LCM最小サポートの計算
	# マニュアルの式(9)
	def calSigma(self,minPos,minGR,posCnt,negCnt):
		omegaF=self.calOmega(posCnt)
		beta=minPos
		w=float(negCnt)/float(posCnt)

		#print("omegaF="+str(omegaF))
		#print("minPos="+str(minPos))
		#print("beta="+str(beta))
		#print("posCnt="+str(posCnt))
		#print("negCnt="+str(negCnt))
		#print("minGR="+str(minGR))
		#print("w="+str(w))

		sigma=int(beta*(omegaF-w/minGR))  # 切り捨て
		if sigma<=0:
			sigma=1 
		return sigma

	def __init__(self,db,outtf=True):
		self.size  = None
		self.pFile = None
		self.tFile = None
		self.temp  = mtemp.Mtemp()
		self.db    = db # 入力データベース
		self.file  = self.temp.file()
		self.outtf = outtf
		self.weightFile = {}
		self.posWeight  = {}
		self.sigma      = {}
		self.msgoff = True
		
		items      = self.db.items
		for cName,posSize in db.clsNameRecSize.items(): 
			self.weightFile[cName] = self.temp.file()
			self.posWeight[cName]  = self.calOmega(posSize)
			f =   nm.mcut(nfno=True,f=self.db.clsFN,i=self.db.cFile)
			f <<= nm.mchgstr(nfn=True,f=0,O=-1,o=self.weightFile[cName],c="%s:%s"%(cName,self.posWeight[cName]))
			f.run()

		# アイテムをシンボルから番号に変換する。
		f = nm.mjoin(k=self.db.itemFN,K=items.itemFN,i=self.db.file,m=items.file,f=items.idFN)
		f <<= nm.mcut(f=self.db.idFN+","+items.idFN)
		f <<= nm.mtra(k=self.db.idFN,f=items.idFN)
		f <<= nm.mcut(f=items.idFN,nfno=True,o=self.file)
		f.run()


	# 各種パラメータを与えて列挙を実行
	def enumerate(self,eArgs):

		pFiles=[]
		tFiles=[]
		tf=mtemp.Mtemp()
		for cName,posSize in self.db.clsNameRecSize.items(): 
			negSize=self.db.traSize-posSize
			if "minGR" in eArgs :
				self.minGR = eArgs["minGR"]
			else:
				minProb = eArgs["minProb"] if ( "minProb" in eArgs ) else 0.5
				if "uniform" in eArgs and eArgs["uniform"] == True:
					self.minGR = (minProb/(1-minProb)) * (self.db.clsSize-1) # マニュアルの式(4)
				else:
					self.minGR = (minProb/(1-minProb)) * (float(negSize)/float(posSize)) # マニュアルの式(4)


			# 最小サポートと最小サポート件数
			# s=0.05
			# s=c1:0.05,c2:0.06
			# S=10
			# S=c1:10,c2:15
			if "minCnt" in eArgs :
				# minCnt(S=をクラスごとできるようにしている形跡：使われない)
				if isinstance(eArgs["minCnt"],dict): 
					self.minPos = eArgs["minCnt"][cName]
				else:
					self.minPos = eArgs["minCnt"]
			else:
				# minSup(s=をクラスごとできるようにしている形跡：使われない)
				if isinstance(eArgs["minSup"],dict):
					self.minPos = int(eArgs["minSup"][cName] * float(posSize) + 0.99)
				else:
					self.minPos = int(eArgs["minSup"] * float(posSize) + 0.99)

			# 最大サポートと最大サポート件数
			if "maxCnt" in eArgs:
				# maxSup(Sx=をクラスごとできるようにしている形跡：使われない)
				if isinstance(eArgs["maxCnt"],dict):
					self.maxPos = eArgs["maxCnt"][cName]
				else:
					self.maxPos = eArgs["maxCnt"]

			elif "maxSup" in eArgs:
				# maxSup(sx=をクラスごとできるようにしている形跡：使われない)
				if isinstance(eArgs["maxSup"],dict):
					self.maxPos = int(eArgs["maxSup"][cName] * float(posSize) + 0.99)
				else:
					self.maxPos = int(eArgs["maxSup"] * float(posSize) + 0.99)
			else:

				self.maxPos = None


			self.sigma[cName] = self.calSigma(self.minPos,self.minGR,posSize,negSize)

			# lcmのパラメータ設定と実行
			# 頻出パターンがなかった場合、lcm出力ファイルが生成されないので
			# そのときのために空ファイルを生成しておいく。
			lcmout = tf.file() # lcm出力ファイル
			with open(lcmout, "w") as efile:
				pass


			runPara={}			

			if self.msgoff:
				runPara["type"] = eArgs["type"]+"IA_"
			else:
				runPara["type"] = eArgs["type"]+"IA"

			#if self.maxPos: #rubyだとif @maxCntなってる（どこにも設定されてないので）動いてないはず
			if self.maxPos:

				runPara["U"] = self.calSigma(self.maxPos,self.minGR,posSize,negSize)

			if "minLen" in eArgs:
				runPara["l"] = str(eArgs["minLen"])

			if "maxLen" in eArgs:
				runPara["u"] = str(eArgs["maxLen"])

			runPara["w"] = self.weightFile[cName]

			runPara["i"] = self.file

			runPara["sup"] = str(self.sigma[cName])

			runPara["o"] = lcmout

			# lcm実行
			#MCMD::msgLog("#{run}")
			#TAKE::run_lcm(run)
			#print(self.sigma)
			#print(runPara)
			#MCMD::msgLog("output patterns to CSV file ...")

			extTake.lcm(runPara)

			pFiles.append(self.temp.file())

			transle = tf.file()
			extTake.lcmtrans(lcmout,"e",transle)

			f =   nm.mdelnull(f="pattern",i=transle)
			f <<= nm.mcal(c='round(${countN},1)',a="neg")
			f <<= nm.mcal(c='round(${countP}/%s,1)'%(self.posWeight[cName]),a="pos")
			f <<= nm.mdelnull(f="pattern") #いる？
			f <<= nm.msetstr(v=cName,a="class")
			f <<= nm.msetstr(v=posSize,a="posTotal")
			f <<= nm.msetstr(v=self.minGR,a="minGR")
			f <<= nm.mcut(f="class,pid,pattern,size,pos,neg,posTotal,minGR",o=pFiles[-1])
			f.run()

			#s = nutil.mrecount(i=self.file)
			#MCMD::msgLog("the number of contrast patterns on class `#{cName}' enumerated is #{s}")

			if self.outtf :
				# トランザクション毎に出現するパターンを書き出す
				#MCMD::msgLog("output tid-patterns ...")
				tFiles.append(self.temp.file())
				xxw= tf.file()
				
				xxw =   nm.mcut(f=self.db.idFN,i=self.db.file)
				xxw <<= nm.muniq(k=self.db.idFN)
				xxw <<= nm.mnumber(S=0,a="__tid",q=True)

				translt = self.temp.file()
				extTake.lcmtrans(lcmout,"t",translt)

				f =   nm.mjoin(k="__tid",m=xxw,f=self.db.idFN,i=translt)
				f <<= nm.msetstr(v=cName,a="class")
				f <<= nm.mcut(f=self.db.idFN+",class,pid",o=tFiles[-1])
				f.run()


		# クラス別のパターンとtid-pidファイルを統合して最終出力
		self.pFile = self.temp.file()
		self.tFile = self.temp.file()

		# パターンファイル併合
		xxpCat = tf.file()
		f =   nm.mcat(i=",".join(pFiles))
		f <<= nm.msortf(f="class,pid")
		f <<= nm.mnumber(s="class,pid",S=0,a="ppid",o=xxpCat)
		f.run()

		# パターンファイル計算
		items=self.db.items
		f =   nm.mcut(f="class,ppid:pid,pattern,size,pos,neg,posTotal,minGR",i=xxpCat)
		f <<= nm.msetstr(v=self.db.traSize,a="total")
		f <<= nm.mcal(c='${total}-${posTotal}',a="negTotal") # negのトータル件数
		f <<= nm.mcal(c='${pos}/${posTotal}',a="support") # サポートの計算
		f <<= nm.mcal(c='if(${neg}==0,1.797693135e+308,(${pos}/${posTotal})/(${neg}/${negTotal}))',a="growthRate")

		if "uniform" in eArgs and eArgs["uniform"] == True :
			f <<= nm.mcal(c='(${pos}/${posTotal})/(${pos}/${posTotal}+(%s-1)*${neg}/${negTotal})'%(self.db.clsSize),a="postProb")
		else:
			f <<= nm.mcal(c='${pos}/(${pos}+${neg})',a="postProb")

		f <<= nm.msel(c='${pos}>=%s&&${growthRate}>=${minGR}'%(self.minPos)) # minSupとminGRによる選択
		f <<= nm.mvreplace(vf="pattern",m=items.file,K=items.idFN,f=items.itemFN)
		f <<= nm.mcut(f="class,pid,pattern,size,pos,neg,posTotal,negTotal,total,support,growthRate,postProb")
		f <<= nm.mvsort(vf="pattern")
		f <<= nm.msortf(f="class%nr,postProb%nr,pos%nr",o=self.pFile)
		f.run()

		# アイテムを包含している冗長なタクソノミを削除
		if items.taxonomy:
			#MCMD::msgLog("reducing redundant rules in terms of taxonomy ...")
			##ここは後で
			zdd=VSOP.constant(0)
			dt = nm.mcut(i=self.pFile,f="pattern")

			for fldVal in dt:
				zdd=zdd+VSOP.itemset(fldVal[0])

			zdd=self.reduceTaxo(zdd,self.db.items)

			xxp1=tf.file()
			xxp2=tf.file()
			xxp3=tf.file()
			zdd.csvout(xxp1)
			
			nm.mcut(nfni=True,f="1:pattern",i=xxp1).mvsort(vf="pattern").msortf(f="pattern",o=xxp2).run()
			nm.msortf(f="pattern",i=self.pFile).mcommon(k="pattern",m=xxp2).msortf(f="class%nr,postProb%nr,pos%nr",o=xxp3).run()
			shutil.move(xxp3,self.pFile)

		

		if self.outtf:
			# 列挙されたパターンを含むtraのみ選択するためのマスタ
			xxp4=nm.mcut(f="class,pid",i=self.pFile)
			f =   nm.mcat(i=",".join(tFiles))
			f <<= nm.mjoin(k="class,pid",m=xxpCat,f="ppid") # 全クラス統一pid(ppid)結合
			f <<= nm.mcommon(k="class,ppid",K="class,pid",m=xxp4) # 列挙されたパターンの選択
			f <<= nm.mcut(f=self.db.idFN+",class,ppid:pid")
			f <<= nm.msortf(f=self.db.idFN+",class,pid",o=self.tFile)
			f.run()

		#self.size = nutil.mrecount(i=self.pFile)
		#MCMD::msgLog("the number of emerging patterns enumerated is #{@size}")


	def output(self,outpath):
		nm.mfldname(q=True,i=self.pFile,o=outpath+"/patterns.csv").run()
		#shutil.move(self.pFile,outpath+"/patterns.csv")
		if self.outtf:
			nm.mfldname(q=True,i=self.tFile,o=outpath+"/tid_pats.csv").run()
			#shutil.move(self.tFile,outpath+"/tid_pats.csv")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/

import os
import glob
import time

#= 一時ファイル名を扱うクラス (内容確認すること)
#
#  一時ファイル名の生成と、そのファイルの(自動)削除を行うクラス*。
#  一時ファイル名はfileもしくはpipeメソッドを呼び出すたびに重複なく生成される。
#  fileメソッドでは、ファイル名が生成するだけで、実ファイルは生成されない。
#  一方でpipeメソッドでは、mkfifoコマンドにより名前付きパイプファイルが生成される。
#
#  一時ファイル名の命名規則は以下の通り。
#   "#{@path}/__MTEMP_#{@pid}_#{@oid}_#{@seq}"
#
#  @pid : プロセスID ($$)
#  @oid : オブジェクトID (self.object_id)
#  @seq : オブジェクト内の通し番号 (自動採番で1から始まる)
#  @path   : 以下の優先順位で決まる。
#       1) Mtemp.newの第1引数で指定された値*
#       2) KG_TmpPath環境変数の値
#       3) TMP環境変数の値
#       4) TEMP環境変数の値
#       5) "/tmp"
#       6) "." (カレントパス)
#
#  注*) new第1引数でパス名を明示的に指定した場合、GC時に自動削除されない。
#
#=== メソッド:
# file : 一時ファイル名を返す
# path : 一時ファイル名を格納するパスを返す
# rm   : 実行時点までに生成した一時ファイルを全て削除する。
#
#=== 例1
#  基本利用例
# ------------------------------------------------------
#  require 'mtools'
#
#  tmp=MCMD::Mtemp.new
#  fName1=tmp.file
#  fName2=tmp.file
#  fName3=tmp.file("./xxa")
#  puts fName1 # -> /tmp/__MTEMP_60637_2152301760_0
#  puts fName2 # -> /tmp/__MTEMP_60637_2152301760_1
#  puts fName3 # -> ./xxa
#  File.open(fName1,"w"){|fp| fp.puts "temp1"}
#  File.open(fName2,"w"){|fp| fp.puts "temp2"}
#  File.open(fName3,"w"){|fp| fp.puts "temp3"}
#  # tmpがローカルのスコープを外れると
#  # GCが発動するタイミングで一時ファイルも自動的に削除される。
#  # ただし、fName3は一時ファイル名を直接指定しているの削除されない。
# ------------------------------------------------------
#
#===例2:
#  全ての一時ファイルが自動削除されない例
# ------------------------------------------------------
#  require 'mtools'
#
#  # コンストラクタでパスを指定すると自動削除されない。
#  # (rmメソッドにより削除することはできる。)
#  tmp=MCMD::Mtemp.new(".")
#  fName=tmp.file
#  File.open(fName,"w"){|fp| fp.puts "temp"}
#  # tmpがローカルのスコープを外れGCが発動しても
#  # 一時ファイルは削除されない。
# ------------------------------------------------------
#
#=== 例3:
#  名前付きパイプ名の生成
# ------------------------------------------------------
#  require 'mtools'
#
#  tmp=MCMD::Mtemp.new
#  pName=tmp.pipe
#  system("ls -l #{pName}") # この段階で名前付きパイプファイルが作成されている。
# ->  prw-r--r--  1 user  group  0  7 19 12:20 /tmp/__MTEMP_60903_2152299720_0
#  system("echo 'abc' > #{pName} &") # バックグラウンド実行でnamed pipeに書き込み
#  system("cat <#{pName} &")         # バックグラウンド実行でnamed pipeから読み込み
#  # tmpがローカルのスコープを外れると
#  # GCが発動するタイミングで全ての一時ファイルは自動削除される。
# ------------------------------------------------------
class Mtemp(object):

	def __init__(self,path=None) :
		self.gcRM=True
		defenv = os.environ

		if path :
			self.path=path
			self.gcRM=False

		elif "KG_TmpPath" in defenv :
			self.path=defenv["KG_TmpPath"]

		elif "TMP" in defenv :
			self.path=ENV["TMP"]

		elif "TEMP" in defenv :
			self.path=ENV["TEMP"]

		elif os.access("/tmp",os.W_OK) :
			self.path="/tmp"

		elif os.access(".",os.W_OK) :
			self.path="."

		else:
			raise("no writable temporal directory found")


		self.pid  = os.getpid()
		self.oid  = id(self)

		self.seq = 0 # オブジェクト内通し連番


	# ワークファイルを全て消す
	def delAllFiles(self,path,pid,oid):
		if  self.pid == os.getpid() and self.oid == id(self):

			for dn in glob.glob("%s/__MTEMP_%s_%s_*"%(path,self.pid,self.oid)):
				os.system("rm -rf %s"%(dn))

	def __del__(self):
		self.delAllFiles(self.path,self.pid,self.oid)		

	def mkname(self):
		ut = time.time()
		return "%s/__MTEMP_%s_%s_%s_%s"%(self.path,self.pid,self.oid,self.seq,str(ut))

	#== 一時ファイル名の取得
	# 返値: 一時ファイル名(String)
	#
	# 以下のフォーマットで一時ファイル名を生成する。
	# seqはカウントアップされる。
	# フォーマット: "#{@path}/__MTEMP_#{@pid}_#{@oid}_#{@seq}"
	# nameが指定されれば(@path以外に)GCで削除しなくなる。
	def file(self,name=None):
		n=None
		if name==None:
			n= self.mkname()
			self.seq += 1
		else:
			n=name

		return n

	#== 一時ファイル名(名前付きパイプ)の取得 deprecated
	# 返値: 一時ファイル名(String)
	#
	# 以下のフォーマットで名前付きパイプの一時ファイル名を生成する。
	# @seqはカウントアップされる。
	# フォーマット: "#{@path}/__MTEMP_#{@pid}_#{@oid}_#{@seq}"
	# nameが指定されれば(@path以外に)GCで削除しなくなる。
	def pipe(self,name=None):
		n=None
		if name==None :
			n=self.mkname()
			self.seq += 1
		else :
			n=name

		os.system("mkfifo %s"%(n))
		return n

	#== 一時ファイルの出力パスの取得
	# 返値: 一時ファイルを出力パス名(String)
	def path(self):
		return self.path

 	#=== 一時ファイルの削除
 	# 以下のコマンドを実行することで一時ファイルを全て削除する。
	# system "rm -rf #{path}/__MTEMP_#{@pid}_#{@oid}_*"
	def rm(self):
		self.delAllFiles(self.path,self.pid,self.oid)

	# ワークファイル強制削除#{path}
	def forceDel(self,save=False):
		for fn in glob.glob("%s/__MTEMP_*"%(self.path)):
			if save :
				os.system("tar cvfzP %s.tar.gz  %s"%(fn,fn))

			os.system("rm -rf %s"%(fn))


#==============================================      
#以下、サンプルコード
#==============================================      
if __name__ == '__main__':
	import os
	tmp=Mtemp()
	fName1=tmp.file()
	fName2=tmp.file()
	fName3=tmp.file("./xxa")
	print(fName1)
	print(fName2)
	print(fName3)

	with open(fName1,'w') as f:
		f.write("temp1\n")
	with open(fName2,'w') as f:
		f.write("temp2\n")
	with open(fName3,'w') as f:
		f.write("temp3\n")

	tmp=Mtemp(".")
	fName=tmp.file()

	with open(fName,'w') as f:
		f.write("temp\n")

	tmp=Mtemp()
	pName=tmp.pipe()
	os.system("ls -l %s"%(pName))
	os.system("echo 'abc' > %s &"%(pName))
	os.system("cat <%s &"%(pName))

# サンプルコード
#==============================================




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Msep(NysolMOD_CORE): 

	_kwd ,_inkwd,_outkwd = n_core.getparalist("msep",3)
	_disabled_ouputlist = True

	def __init__(self,*args, **kw_args) :
		super(Nysol_Msep,self).__init__("msep",nutil.args2dict(args,kw_args,Nysol_Msep._kwd))


def msep(self,*args, **kw_args):
	return Nysol_Msep(nutil.args2dict(args,kw_args,Nysol_Msep._kwd)).addPre(self)

setattr(NysolMOD_CORE, "msep", msep)



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Readlist(NysolMOD_CORE):
	_kwd ,_inkwd,_outkwd = n_core.getparalist("readlist",3)


	def __init__(self,*args, **kw_args) :
		if len(args) != 1 :
			print("arge only one")
			return None

		if isinstance(args[0],list) :
			kw_args["i"] = args[0]
		else :
			print("unsuport type")
			return None
		super(Nysol_Readlist,self).__init__("readlist",nutil.args2dict((),kw_args,Nysol_Readlist._kwd))




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Writelist(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd  = n_core.getparalist("writelist",3)
	_kwd[0].append("dtype")
	_kwd[1].append("header")

	def __init__(self,*args, **kw_args) :
		super(Nysol_Writelist,self).__init__("writelist",nutil.args2dict(args,kw_args,Nysol_Writelist._kwd,uk="o"))


def writelist(self,*args, **kw_args):
	return Nysol_Writelist(nutil.args2dict(args,kw_args,Nysol_Writelist._kwd,uk="o")).addPre(self)

setattr(NysolMOD_CORE, "writelist", writelist)

